import pyglet
import Box2D.Box2D as b2
import math
import pickle
import squirtle
import os
from math import cos, sin, radians

data_path = 'data\\char\\'
images = {}
def LoadImages():
    global images
    squirtle.SVG._disp_list_cache = {} # clear display list cache
    images = {}
    files = os.listdir(data_path)
    for f in files:
        images[f] = squirtle.SVG(data_path + f, 'left', 'bottom', invert_y=True)
    print images


class DrawUnit(object):

    def __init__(self, fdraw, layer):
        self.fdraw = fdraw
        self.layer = layer
    
    def __call__(self):
        self.fdraw()

class Keyframe:
    def __init__(self, d=0, theta=0, ox=0, oy = 0):
        self.angle = theta
        self.distance = d
        self.offset_x = ox
        self.offset_y = oy


    def copy(self):
        return Keyframe(self.distance, self.angle, self.offset_x, self.offset_y)

class Joint:
    def __init__(self, parent=None):

        self.keyframes = []
        self.layer = 0

        #joint constraints
        self.min_angle = None
        self.max_angle = None

        self.attached_item = None
        self.reverse = False

        self.parent = parent
        self.children = []
        if self.parent:
            self.parent.AddBone(self)
        else:
            #make sure there's at least one keyframe
            #when we create the root
            self.keyframes.append(Keyframe())
    
    def All(self):
        l = [self]
        self._all(l)
        return l

    def Delete(self):
        if self.parent:
            self.parent.RemoveBone(self)

    def RemoveBone(self, bone):
        self.children.remove(bone)

    def _all(self, l):
        l.extend(self.children)
        for c in self.children:
            c._all(l)

    def AddBone(self, b):
        b.keyframes = [Keyframe() for x in xrange(len(self.keyframes))]
        print b.keyframes
        self.children.append(b)
       
    def AddKeyframe(self, index):
        new_frame = self.keyframes[index].copy()
        self.keyframes.insert(index+1, new_frame)

    def RemoveKeyframe(self, index):
        del self.keyframes[index]

    def CalculateEndpoint(self, keyframe):
        a = self.WorldAngle(keyframe)
        d = self.keyframes[keyframe].distance
        v = b2.b2Vec2(cos(a)*d, sin(a)*d)
        v.x += self.keyframes[keyframe].offset_x
        v.y += self.keyframes[keyframe].offset_y
        if self.parent:
            v = v + self.parent.CalculateEndpoint(keyframe)
        return v

    def WorldAngle(self, keyframe):
        a = 0
        if self.parent: 
            a = self.parent.WorldAngle(keyframe)
        return self.keyframes[keyframe].angle + a
        

    def HasChildren(self):
        return len(self.children) > 0

    def SetEndpoint(self, x, y, keyframe, set_all_keyframes=False, keep_length = False):
        if not self.parent:
            a = b2.b2Vec2(0,0)
            wa = 0
        else:
            a = self.parent.CalculateEndpoint(keyframe)
            wa = self.parent.WorldAngle(keyframe)
        
        b = b2.b2Vec2(x, y)
        d = b - a

        n = d.copy()
        n.Normalize()
        
        if set_all_keyframes:
            for kf in self.keyframes:
                kf.angle = math.atan2(n.y, n.x) - wa
                if not keep_length:
                    kf.distance = d.Length()
        else:
            kf = self.keyframes[keyframe]
            kf.angle = math.atan2(n.y, n.x) - wa
            if not keep_length:
                kf.distance = d.Length()


    def DrawPose(self, keyframe):
        things_to_draw = self.PreDraw(keyframe)
        sorted_things_to_draw = sorted(things_to_draw, lambda x,y: x.layer-y.layer, reverse=True)
        for st in sorted_things_to_draw: st()

    def PreDraw(self, keyframe):
        du = []
    
        a = self.CalculateEndpoint(keyframe)
        
        if self.attached_item:
            svg = images[self.attached_item]
            du.append(DrawUnit(
                            lambda: svg.draw(a.x, a.y, angle=(self.WorldAngle(keyframe) * (180/math.pi) - 90 )),
                            self.layer))

        for jc in self.children:
            x = jc.PreDraw(keyframe)
            du.extend(x)
        return du
        
def Load(filename):
    return pickle.load(open(filename, 'rb'))

LoadImages()