#/usr/bin/env python
import pyglet
import pyglet.gl
import ctypes
import settings

gl = pyglet.gl

class DisplayList(object):
    def __enter__(self):
        self._display_list = gl.glGenLists(1)
        gl.glNewList(self._display_list, gl.GL_COMPILE)
        return _self.display_list

    def __exit__(self, type, value, traceback):
        gl.glEndList()

class FramebufferObject(object):
    def __init__(self, w, h):
        x = ctypes.c_ulong()
        px = ctypes.pointer(x)
        gl.glGenFramebuffersEXT(1, px)
        self.id = x.value

        self.bind()
        self.texture = pyglet.image.Texture.create(w, h)
        gl.glBindTexture(gl.GL_TEXTURE_2D, self.texture.id)
        gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MAG_FILTER, gl.GL_NEAREST);
        gl.glTexParameteri(gl.GL_TEXTURE_2D, gl.GL_TEXTURE_MIN_FILTER, gl.GL_NEAREST);

        self.attach(self.texture)

        error = gl.glCheckFramebufferStatusEXT(gl.GL_FRAMEBUFFER_EXT)
        print error
        if error == gl.GL_FRAMEBUFFER_UNSUPPORTED_EXT:
            raise "Framebuffer unsupported"
        elif error == gl.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:
            raise "Incomplete attachment"
        elif error == gl.GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT:
            raise "Incomplete dimensions"
        elif error == gl.GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT:
            raise "Incomplete draw buffer"
        elif error == gl.GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT:
            raise "Incomplete formats"
        elif error == gl.GL_FRAMEBUFFER_COMPLETE_EXT:
            print "Hooray"
        elif error == gl.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT:
            raise "Missing attachment"
        elif error == gl.GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT:
            raise "Incomplete read buffer"
        else:
            raise "???"

        self.unbind()
        

    def bind(self):
        gl.glBindFramebufferEXT(gl.GL_FRAMEBUFFER_EXT, self.id);

    def unbind(self):
        gl.glBindFramebufferEXT(gl.GL_FRAMEBUFFER_EXT, 0);

    def attach(self, texture):
        if isinstance(texture, pyglet.image.Texture):
            texture_id = texture.id
        else:
            texture_id = texture

        gl.glFramebufferTexture2DEXT(gl.GL_FRAMEBUFFER_EXT, gl.GL_COLOR_ATTACHMENT0_EXT, gl.GL_TEXTURE_2D, texture_id, 0)

    def detach(self):
        gl.glFramebufferTexture2DEXT(gl.GL_FRAMEBUFFER_EXT, gl.GL_COLOR_ATTACHMENT0_EXT, gl.GL_TEXTURE_2D, 0, 0)


class Screen:
    def __init__(self):
        self.width = settings.current.width
        self.height = settings.current.height
    
    def set_viewport(self, width, height, invert=False):
        gl = pyglet.gl.gl
        gl.glViewport(0, 0, width, height)
        self.set_projection(width, height, invert)
    
    def set_projection(self, w, h, invert):
        gl.glMatrixMode(gl.GL_PROJECTION)
        gl.glLoadIdentity()
        if invert:
            gl.glOrtho(0, w, h, 0, -1, 1)
        else:
            gl.glOrtho(0, w, 0, h, -1, 1)
        gl.glMatrixMode(gl.GL_MODELVIEW)
    
    
    def save_projection(self):
        gl.glMatrixMode(gl.GL_PROJECTION)
        gl.glPushMatrix()
        gl.glMatrixMode(gl.GL_MODELVIEW)
    
    def restore_projection(self):
        gl.glMatrixMode(gl.GL_PROJECTION)
        gl.glPopMatrix()
        gl.glMatrixMode(gl.GL_MODELVIEW)

screen = Screen()


def draw_image(x, y, w, h, texture):
    gl.glEnable(texture.target)        # typically target is GL_TEXTURE_2D
    gl.glBindTexture(texture.target, texture.id)
    gl.glColor4f(1,1,1,1)
    gl.glPushMatrix()
    gl.glTranslatef(x, y, 0)
    gl.glBegin(gl.GL_QUADS)
    gl.glTexCoord2f(0, 0)
    gl.glVertex2f(0, 0)
    gl.glTexCoord2f(1, 0)
    gl.glVertex2f(w, 0)
    gl.glTexCoord2f(1, 1)
    gl.glVertex2f(w, h)
    gl.glTexCoord2f(0, 1)
    gl.glVertex2f(0, h)
    gl.glEnd()
    gl.glPopMatrix()
    gl.glDisable(gl.GL_TEXTURE_2D)