#/usr/bin/env python
import pyglet
import pyglet.gl
import ctypes
import grfx
import squirtle.shader as shader
import time
import math


gl = pyglet.gl


class IsoGrid(object):

    isoshader = None
    blobshader = None
    width = 1024
    height = 1024

    def __init__(self, mainwindow):
        if not self.isoshader:
            self.isoshader = shader.MakeProgramFromSource(
                                        'isoshader',
                                        open('src/vertex.glsl', 'rt').read(),
                                        open('src/isoshader.glsl', 'rt').read())
            self.isoshader.stop()

        if not self.blobshader:
            self.blobshader = shader.MakeProgramFromSource(
                                        'blob',
                                        open('src/vertex.glsl', 'rt').read(),
                                        open('src/blobshader.glsl', 'rt').read())
            self.blobshader.stop()

        self.draw_commands = []
        self.fbo = grfx.FramebufferObject(self.width, self.height)
        self.mainwindow = mainwindow
        self.redraw_surface_texture()

    def update(self, dt, camera, scale):
        self.redraw_surface_texture(camera, scale)

    def redraw_surface_texture(self, camera=None, scale=1.0):
        
        self.fbo.bind()
        grfx.screen.set_viewport(self.width, self.height, True)
                
        gl.glPushMatrix()
        
        gl.glTranslatef(0, self.height, 0)
        gl.glScalef(1, -1, 1)
        
        gl.glClearColor(0.0, 0.0, 0.0, 0.0)
        gl.glClear(gl.GL_COLOR_BUFFER_BIT)

        self.blobshader.use()

        gl.glPushMatrix()
        if camera:
            gl.glTranslatef(-camera.x, -camera.y, 0)
        gl.glScalef(scale, scale, 1.0)
        for cmd in self.draw_commands:
            cmd()
        gl.glPopMatrix()
        
        self.blobshader.stop()

        gl.glPopMatrix()

        self.fbo.unbind()
