import pyglet
import settings
import squirtle
import isomap
import grfx
import scene, level


from stats import stats
from grfx import screen

# Use psycho if it's installed. Framerate++!
try: 
    import psycho
    psycho.full()
except:
    pass

gl = pyglet.gl.gl

class MainWindow(pyglet.window.Window):
    
    def init(self):
        try:
            pyglet.font.add_directory('data/fonts/')
            print "Fonts loaded"
        except Exception, e:
            print "Could not load font: " + str(e)
        
        squirtle.setup_gl()

        gl.glClearColor(1,1,1,1)
        gl.glEnable(gl.GL_LINE_SMOOTH)
        gl.glHint(gl.GL_LINE_SMOOTH_HINT, gl.GL_NICEST);
        gl.glLineWidth(2)

        self.fps_display = pyglet.clock.ClockDisplay()
        pyglet.clock.set_fps_limit(100)
        
        cursor = self.get_system_mouse_cursor(pyglet.window.Window.CURSOR_CROSSHAIR)
        self.set_mouse_cursor(cursor)    

        self.scene_manager = scene.SceneManager([level.MainScene("Main", self)])
        
        self.keys = pyglet.window.key.KeyStateHandler()
        self.push_handlers(self.keys)

    def on_resize(self, width, height):
        screen.set_viewport(width, height)
    
    def on_mouse_press(self, x, y, button, modifiers):
        self.scene_manager.current_scene.on_mouse_press(x, y, button, modifiers)
    
    def on_mouse_release(self, x, y, button, modifiers):
        self.scene_manager.current_scene.on_mouse_release(x, y, button, modifiers)
        
    def on_mouse_motion(self, x, y, dx, dy):
        self.scene_manager.current_scene.on_mouse_motion(x, y, dx, dy)
    
    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        self.scene_manager.current_scene.on_mouse_drag(x, y, dx, dy, button, modifiers)
    
    def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
        self.scene_manager.current_scene.on_mouse_scroll(x, y, scroll_x, scroll_y)
        
    def on_key_press(self, symbol, modifiers):
        super(MainWindow, self).on_key_press(symbol, modifiers)
        self.scene_manager.current_scene.on_key_press(symbol, modifiers)
    
    def step(self):
        dt = pyglet.clock.tick()
        self.scene_manager.current_scene.update(dt)
        stats.update('dt', dt)
    
    def redraw(self):
        
        screen.set_viewport(screen.width, screen.height)
        gl = pyglet.gl.gl
        gl.glClearColor(0.15,0.15,0.25,1)
        self.clear()

        self.scene_manager.current_scene.draw()
        self.scene_manager.current_scene.draw_ui()

        #stats.draw()
        #Gets fps and draw it
        #if settings.current.show_framerate:
            #self.fps_display.draw()
        self.flip()


def main():
    config = pyglet.gl.Config(sample_buffers=1, samples=4)
    if settings.current.fullscreen:
        mw = MainWindow(vsync=0,
                          fullscreen=settings.current.fullscreen)
    else:
        mw = MainWindow(width=settings.current.width,
                          height=settings.current.height,
                          config=config,
                          vsync=0,
                          fullscreen=settings.current.fullscreen)
    mw.init()
    
    while not mw.has_exit:
        mw.dispatch_events()
        
        #step the game world
        mw.step()
        
        #draw game world here
        mw.redraw()
        

