import pyglet
import Box2D.Box2D as b2
import math
import pickle
import os
import os.path
import sys
from math import cos, sin, radians
from pyglet.window import key, mouse
import squirtle
from animation import *

gl = pyglet.gl

data_path = 'data/char/'

SCALE = 2

images = {}
def LoadImages():
    global images
    squirtle.SVG._disp_list_cache = {} # clear display list cache
    images = {}
    files = os.listdir(data_path)
    for f in files:
        images[f] = squirtle.SVG(data_path + f, 'left', 'bottom', invert_y=True)
    print images



class State:
    def __init__(self, filename=None):
        self.root = Joint()
        self.filename = filename
        
        if self.filename != None and os.path.exists(filename):
            try:
                data = pickle.load(open(filename, 'rb'))
                self.root = data
            except Exception, e:
                tool_label.text = str(e)
        
        self.selected = self.root
        self.camera = b2.b2Vec2(256, 200)
        self.mode = None
        self.lock_length = False
        self.keyframe = 0
        self.nkeyframes = len(self.root.keyframes)
        self.baseline = 40
        squirtle.setup_gl()

    def save(self):
        if not self.filename:
            self.filename = 'temp.skel'
        pickle.dump(self.root, open(self.filename, 'wb'), 0)
        tool_label.text = 'Saved ' + self.filename

    def mouse_moved(self, x, y, dx, dy):
        if self.mode == 'add' or self.mode == 'drag':
            self.place(x,y)

    def select(self, x, y):
        all = self.root.All()
        mouse = b2.b2Vec2(x - self.camera.x, y - self.camera.y)
        for j in all:
            p = j.CalculateEndpoint(state.keyframe)
            if (p - mouse).Length() < 8:
                self.selected = j
                self.mode = 'drag'
                self.update_selected_text()
                return
   
    def delete_selected(self):
        if self.selected.parent:
            self.selected.Delete()
            self.selected = self.selected.parent

    def start_placement(self, x, y):
        self.mode = 'add'
        self.selected = Joint(self.selected)
        self.place(x,y)
        print "start", x, y

    def place(self, x, y):
        lock = self.lock_length
        if self.mode == 'drag':
            lock = not lock
        
        set_all_keyframes = False
        if self.mode == 'add':
            set_all_keyframes = True
        
        kf = self.keyframe

        if self.selected == self.root and lock:
            self.selected.keyframes[self.keyframe].offset_x = x - self.camera.x
            self.selected.keyframes[self.keyframe].offset_y = y - self.camera.y
        else:
            self.selected.SetEndpoint(x - self.camera.x, y - self.camera.y, kf, set_all_keyframes, lock)
        self.update_selected_text()

    def update_selected_text(self):
        tool_label.text = 'Layer: %d, Angle: %d, Length: %d' % (self.selected.layer, int(self.selected.keyframes[state.keyframe].angle * (180/math.pi)), int(self.selected.keyframes[state.keyframe].distance))

    def finish_placement(self, x, y):
        self.mode = None
        self.place(x,y)
        print "final", x, y

    def insert_keyframe(self):
        all = self.root.All()
        for x in all:
            x.AddKeyframe(self.keyframe)
        self.nkeyframes += 1

    def delete_current_keyframe(self):
        all = self.root.All()
        for x in all:
            x.RemoveKeyframe(self.keyframe)
        
        self.nkeyframes -= 1
        if self.keyframe >= self.nkeyframes:
            self.keyframe = self.nkeyframes - 1

    def previous_keyframe(self):
        self.keyframe -= 1
        if self.keyframe < 0: self.keyframe = 0

    def next_keyframe(self):
        self.keyframe += 1
        if self.keyframe >= self.nkeyframes:
            self.keyframe = self.nkeyframes-1


class ImageMenu(object):
    def __init__(self):
        self.items = []
        self.cursor = 0
        self.is_open = False

    def open(self):
        self.is_open = True
        files = sorted(images.keys())
        files.insert(0, None)
        
        for i, f in enumerate(files):
            self.items.append(pyglet.text.Label(str(f),
                          font_name='Consolas',
                          font_size=8,
                          x=0, y= 480 - i*10,
                          anchor_x='left', anchor_y='top'))
    
    def next(self):
        self.cursor += 1

    def prev(self):
        self.cursor -= 1

    def draw(self):
        for i, label in enumerate(self.items):
            if i == self.cursor:
                label.x = 10
            else:
                label.x = 0
            label.draw()

    def selected(self):
        return self.items[self.cursor].text

    def close(self):
        self.is_open = False


LoadImages()

window = pyglet.window.Window(width=1024, height=768)
tool_label = pyglet.text.Label('Info',
                          font_name='Consolas',
                          font_size=16,
                          x=0, y=0,
                          anchor_x='left', anchor_y='bottom')
keyframe_label = pyglet.text.Label('Keyframe',
                          font_name='Consolas',
                          font_size=16,
                          x=0, y=20,
                          anchor_x='left', anchor_y='bottom')


menu = ImageMenu()

@window.event
def on_key_press(symbol, modifiers):
    if symbol == key.LSHIFT:
        state.lock_length = True
    print symbol, modifiers

@window.event
def on_key_release(symbol, modifiers):

    

    if symbol == key.SPACE:
        if not menu.is_open:
            menu.open()
        else:
            menu.close()

    if menu.is_open:
        if symbol == key.R:
            LoadImages()
            menu.open()
        if symbol == key.UP:
            menu.prev()
        elif symbol == key.DOWN:
            menu.next()
        elif symbol == key.ENTER:
            fname = menu.selected()
            if fname == 'None':
                state.selected.attached_item = None
            else:
                state.selected.attached_item = fname
            menu.close()
    else:
        if symbol == key.R:
            LoadImages()
        if symbol == key.LSHIFT:
            state.lock_length = False
        if symbol == key.DELETE:
            state.delete_selected()

        if symbol == key.LEFT:
            state.previous_keyframe()
        if symbol == key.RIGHT:
            state.next_keyframe()
        if symbol == key.I:
            state.insert_keyframe()

        if symbol == key.UP:
            state.selected.layer += 1
            tool_label.text = 'Layer: ' + str(state.selected.layer)

        if symbol == key.DOWN:
            state.selected.layer -= 1
            tool_label.text = 'Layer: ' + str(state.selected.layer)

        if symbol == key.S and modifiers & key.MOD_CTRL:
            print "Saving"
            state.save()

        if symbol == key.DELETE and modifiers & key.MOD_CTRL:
            print "Deleting keyframe"
            state.delete_current_keyframe()


@window.event
def on_mouse_motion(x,y,dx,dy):
    state.mouse_moved(x/SCALE,y/SCALE,dx,dy)

@window.event
def on_mouse_drag(x,y,dx,dy, buttons, modifiers):
    state.mouse_moved(x/SCALE,y/SCALE,dx,dy)

@window.event
def on_mouse_press(x, y, button, modifiers):
    
    if button == mouse.LEFT:
        state.select(x/SCALE, y/SCALE)

    if button == mouse.RIGHT:
        state.start_placement(x/SCALE, y/SCALE)

@window.event
def on_mouse_release(x, y, button, modifiers):
    if button == mouse.LEFT:
        state.mode = ''
    if button == mouse.RIGHT:
        state.finish_placement(x/SCALE, y/SCALE)


def draw_scene():
    
    all = state.root.All()
    
    baseline = min((j.CalculateEndpoint(state.keyframe).y for j in all))

    gl.glPushMatrix()
    gl.glScalef(SCALE, SCALE, 1)
    gl.glTranslatef(state.camera.x, state.camera.y, 0)
    things_to_draw, debug_stuff = draw_joint(state.root)

    sorted_things_to_draw = sorted(things_to_draw, lambda x,y: x.layer-y.layer, reverse=True)
    
    for st in sorted_things_to_draw: st()
    for st in debug_stuff: st()
    

    #draw baseline
    gl.glColor4f(0.0, 0.25, 1.0, 1.0)
    pyglet.graphics.draw(2, pyglet.gl.GL_LINES,
                             ('v2f', (-1000, baseline, 1000, baseline)))
    gl.glPopMatrix()
    tool_label.draw()

    keyframe_label.text = 'Keyframe: %d / %d' % (state.keyframe+1, state.nkeyframes)
    keyframe_label.draw()

def draw_menu():
    menu.draw()

@window.event
def on_draw():
    gl.glClearColor(0.0, 0.25, 0.5, 1)
    window.clear()

    if menu.is_open:
        draw_menu()
    else:
        draw_scene()



def draw_joint(j):
    du = []
    ddu = []
    
    def debug_state():
        gl.glPointSize(6)
        gl.glLineWidth(2)
        if j == state.selected:
            gl.glColor4f(0, 1, 0, 0.75)
        else:
            gl.glColor4f(1, 0, 0, 0.5)
        return True
    
    a = j.CalculateEndpoint(state.keyframe)
        
    if j.attached_item:
        svg = images[j.attached_item]
        du.append(DrawUnit(
                        lambda: svg.draw(a.x, a.y, angle=(j.WorldAngle(state.keyframe) * (180/math.pi) - 90 )),
                        j.layer))
    
    ddu.append(
        lambda: debug_state() and pyglet.graphics.draw(1, pyglet.gl.GL_POINTS,
                                ('v2f', (a.x, a.y)))   )
    if j.parent:
        b = j.parent.CalculateEndpoint(state.keyframe)
        ddu.append(lambda: debug_state() and pyglet.graphics.draw(2, pyglet.gl.GL_LINES,
                             ('v2f', (a.x, a.y, b.x, b.y))))

    for jc in j.children:
        x,y = draw_joint(jc)
        du.extend(x)
        ddu.extend(y)

    return du, ddu

if len(sys.argv) > 1:
    state = State(sys.argv[1])
else:
    state = State()
#window.push_handlers(pyglet.window.event.WindowEventLogger())

pyglet.app.run()